function [Phi, om] = beammodes(geom, mater, mesh, h, nModes)
% beammodes
%   [Phi, om] = beammodes(geom, mater, mesh, h, nModes)

% Copyright 2010 Peter Fiala
% Laboratory of Acoustics
% Dept. of Telecommunications
% Budapest University of Technology and Economics

% properties
B = mater.E * geom.d*h.^3/12;
mu = mater.rho*geom.d*h;

% Assemble FEM matrices
K = sparse(2*(mesh.N+1),2*(mesh.N+1));
M = sparse(2*(mesh.N+1),2*(mesh.N+1));
for i = 1 : mesh.N
    ind = 2*(i-1)+(1:4);
    [k, m] = beamelement(geom.L/mesh.N);
    K(ind,ind) = K(ind,ind) + B(i)*k;
    M(ind,ind) = M(ind,ind) + mu(i)*m;
end

% modal analysis
warning off all;
options.disp = 0;
[Phi, om] = eigs(K, M, nModes+2, 'sm', options);
warning on all;

% sort modes by frequency and exclude rigid body modes
[om, i] = sort(sqrt(diag(om)));
om = om(2+(1:nModes));
Phi = Phi(:,i(2+(1:nModes)));

end

function [k, m] = beamelement(l)
k = [
    12, 6*l, -12, 6*l
    6*l, 4*l^2, -6*l, 2*l^2
    -12, -6*l, 12, -6*l
    6*l, 2*l^2, -6*l, 4*l^2
    ] / l^3;

m = [
    156, 22*l, 54, -13*l
    22*l, 4*l^2, 13*l, -3*l^2
    54, 13*l, 156, -22*l
    -13*l, -3*l^2, -22*l, 4*l^2
    ]*l/420;
end